<?php namespace MapGuesser\Repository;

use Generator;
use SokoWeb\Database\Query\Select;
use MapGuesser\PersistentData\Model\Challenge;
use MapGuesser\PersistentData\Model\Guess;
use MapGuesser\PersistentData\Model\User;
use MapGuesser\PersistentData\Model\Place;
use MapGuesser\PersistentData\Model\PlaceInChallenge;

class GuessRepository
{
    public function getAllByUser(User $user): Generator
    {
        $select = new Select(\Container::$dbConnection);
        $select->where('user_id', '=', $user->getId());

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class);
    }

    public function getAllByUserAndChallenge(User $user, Challenge $challenge): Generator
    {
        $select = new Select(\Container::$dbConnection);
        $select->innerJoin('place_in_challenge', ['place_in_challenge', 'id'], '=', ['guesses', 'place_in_challenge_id']);
        $select->where('user_id', '=', $user->getId());
        $select->where(['place_in_challenge', 'challenge_id'], '=', $challenge->getId());

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class);
    }

    public function getByUserAndPlaceInChallenge(User $user, Challenge $challenge, Place $place): ?Guess
    {
        $select = new Select(\Container::$dbConnection);
        $select->innerJoin('place_in_challenge', ['place_in_challenge', 'id'], '=', ['guesses', 'place_in_challenge_id']);
        $select->where('user_id', '=', $user->getId());
        $select->where(['place_in_challenge', 'challenge_id'], '=', $challenge->getId());
        $select->where(['place_in_challenge', 'place_id'], '=', $place->getId());

        return \Container::$persistentDataManager->selectFromDb($select, Guess::class);
    }

    public function getAllInChallengeByUser(int $userId, Challenge $challenge): Generator
    {
        $select = new Select(\Container::$dbConnection);
        $select->innerJoin('place_in_challenge', ['place_in_challenge', 'id'], '=', ['guesses', 'place_in_challenge_id']);
        $select->where('user_id', '=', $userId);
        $select->where(['place_in_challenge', 'challenge_id'], '=', $challenge->getId());
        $select->orderBy(['place_in_challenge', 'round']);

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class);
    }

    public function getAllInChallenge(Challenge $challenge, array $withRelations = []): Generator
    {
        if (count($withRelations)) {
            $necessaryRelations = ['place_in_challenge'];
            $withRelations = array_unique(array_merge($withRelations, $necessaryRelations));
        }

        $select = new Select(\Container::$dbConnection);
        $select->where(['guesses__place_in_challenge', 'challenge_id'], '=', $challenge->getId());
        $select->orderBy(['guesses__place_in_challenge', 'round']);

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class, true, $withRelations);
    }

    public function getAllInChallengeByRound(int $round, Challenge $challenge, array $withRelations = []): Generator
    {
        if (count($withRelations)) {
            $necessaryRelations = ['place_in_challenge'];
            $withRelations = array_unique(array_merge($withRelations, $necessaryRelations));
        }

        $select = new Select(\Container::$dbConnection);
        $select->where(['guesses__place_in_challenge', 'challenge_id'], '=', $challenge->getId());
        $select->where(['guesses__place_in_challenge', 'round'], '=', $round);

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class, true, $withRelations);
    }

    public function getAllByPlace(Place $place): Generator
    {
        $select = new Select(\Container::$dbConnection);
        $select->innerJoin('place_in_challenge', ['place_in_challenge', 'id'], '=', ['guesses', 'place_in_challenge_id']);
        $select->where(['place_in_challenge', 'place_id'], '=', $place->getId());

        yield from \Container::$persistentDataManager->selectMultipleFromDb($select, Guess::class);
    }
}
